@@ -27,6 +27,7 @@ except ImportError:
     log.info("Evolution support disabled")
 
 class EvoBase(DataProvider.TwoWay):
+    _configurable_ = True
     def __init__(self, sourceURI, *args):
         DataProvider.TwoWay.__init__(self)
         self.defaultSourceURI = sourceURI
@@ -169,8 +170,7 @@ class EvoContactTwoWay(EvoBase):
     def _create_object(self, contact):
         obj = evolution.ebook.EContact(vcard=contact.get_vcard_string())
         if self.book.add_contact(obj):
-            mtime = datetime.datetime.fromtimestamp(obj.get_modified())
-            return conduit.datatypes.Rid(uid=obj.get_uid(), mtime=mtime, hash=mtime)
+            return self._get_object(obj.get_uid()).get_rid()
         else:
             raise Exceptions.SyncronizeError("Error creating contact")
 
